<?php
/**
 * @package     Falang for Joomla!
 * @author      Stéphane Bouey <stephane.bouey@faboba.com> - http://www.faboba.com
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @copyright   Copyright (C) 2011-2021 Faboba. All rights reserved.
 */

// no direct access
defined('_JEXEC') or die;

use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Factory;

/**
 * Falang Tag Plugin
 *
 */

class PlgContentFalangtag extends CMSPlugin
{
	/**
	 * Affects constructor behavior. If true, language files will be loaded automatically.
	 *
	 * @var    boolean
	 * @since  3.1
	 */
	protected $autoloadLanguage = true;

	/**
	 * Plugin that adds a pagebreak into the text and truncates text at that point
	 *
	 * @param   string   $context  The context of the content being passed to the plugin.
	 * @param   object   &$row     The article object.  Note $article->text is also available
	 * @param   mixed    &$params  The article params
	 * @param   integer  $page     The 'page' number
	 *
	 * @return  mixed  Always returns void or true
	 *
	 * @since   1.6
	 */
	public function onContentPrepare($context, &$row, &$params, $page = 0)
	{
        $canProceed = $context === 'com_tags.tag';

        if (!$canProceed)
        {
            return;
        }

        if ($row->type_alias == 'com_content.article'){
            $lang = Factory::getLanguage()->getTag();
            $default_lang = ComponentHelper::getParams('com_languages')->get('site', 'en-GB');

            //nothing to do for default langauge
            if ($lang == $default_lang ){
                return;
            }
            $db = Factory::getDbo();
            $query = $db->getQuery(true);
	        $query->select($db->quoteName(array('id','title','alias','introtext')))
		        ->from('#__content AS c')
		        ->where('c.id = '.$row->content_item_id);
	        $db->setQuery($query);
	        $result = $db->loadObject();
	        if (isset($result)){
		        $row->core_title = $result->title;
		        $row->core_alias = $result->alias;
		        $row->text = $result->introtext;//fulltext not used
	        }
        }

		return true;
	}

}
